@@ -46,8 +46,7 @@ from libcloud.test.compute.test_openstack import OpenStack_2_0_MockHttp
 class OpenStackIdentityConnectionTestCase(unittest.TestCase):
     def setUp(self):
         OpenStackBaseConnection.auth_url = None
-        OpenStackBaseConnection.conn_classes = (OpenStackMockHttp,
-                                                OpenStackMockHttp)
+        OpenStackBaseConnection.conn_class = OpenStackMockHttp
 
     def test_auth_url_is_correctly_assembled(self):
         tuples = [
@@ -209,8 +208,7 @@ class OpenStackIdentityConnectionTestCase(unittest.TestCase):
         self.assertEqual(mocked_auth_method.call_count, 1)
 
     def _get_mock_connection(self, mock_http_class, auth_url=None):
-        OpenStackBaseConnection.conn_classes = (mock_http_class,
-                                                mock_http_class)
+        OpenStackBaseConnection.conn_class = mock_http_class
 
         if auth_url is None:
             auth_url = "https://auth.api.example.com"
@@ -228,7 +226,7 @@ class OpenStackIdentity_2_0_ConnectionTests(unittest.TestCase):
     def setUp(self):
         mock_cls = OpenStackIdentity_2_0_MockHttp
         mock_cls.type = None
-        OpenStackIdentity_2_0_Connection.conn_classes = (mock_cls, mock_cls)
+        OpenStackIdentity_2_0_Connection.conn_class = mock_cls
 
         self.auth_instance = OpenStackIdentity_2_0_Connection(auth_url='http://none',
                                                               user_id='test',
@@ -249,7 +247,7 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
     def setUp(self):
         mock_cls = OpenStackIdentity_3_0_MockHttp
         mock_cls.type = None
-        OpenStackIdentity_3_0_Connection.conn_classes = (mock_cls, mock_cls)
+        OpenStackIdentity_3_0_Connection.conn_class = mock_cls
 
         self.auth_instance = OpenStackIdentity_3_0_Connection(auth_url='http://none',
                                                               user_id='test',
@@ -426,12 +424,36 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
         self.assertTrue(result)
 
 
+class OpenStackIdentity_3_0_Connection_OIDC_access_token_federation_projectsTests(
+        unittest.TestCase):
+    def setUp(self):
+        mock_cls = OpenStackIdentity_3_0_federation_projects_MockHttp
+        mock_cls.type = None
+        OpenStackIdentity_3_0_Connection_OIDC_access_token.conn_class = mock_cls
+
+        self.auth_instance = OpenStackIdentity_3_0_Connection_OIDC_access_token(auth_url='http://none',
+                                                                                user_id='idp',
+                                                                                key='token',
+                                                                                tenant_name='oidc',
+                                                                                domain_name='test_domain')
+        self.auth_instance.auth_token = 'mock'
+
+    def test_authenticate(self):
+        auth = OpenStackIdentity_3_0_Connection_OIDC_access_token(auth_url='http://none',
+                                                                  user_id='idp',
+                                                                  key='token',
+                                                                  token_scope='project',
+                                                                  tenant_name="oidc",
+                                                                  domain_name='test_domain')
+        auth.authenticate()
+
+
 class OpenStackIdentity_3_0_Connection_OIDC_access_tokenTests(
         unittest.TestCase):
     def setUp(self):
         mock_cls = OpenStackIdentity_3_0_MockHttp
         mock_cls.type = None
-        OpenStackIdentity_3_0_Connection_OIDC_access_token.conn_classes = (mock_cls, mock_cls)
+        OpenStackIdentity_3_0_Connection_OIDC_access_token.conn_class = mock_cls
 
         self.auth_instance = OpenStackIdentity_3_0_Connection_OIDC_access_token(auth_url='http://none',
                                                                                 user_id='idp',
@@ -454,7 +476,7 @@ class OpenStackIdentity_2_0_Connection_VOMSTests(unittest.TestCase):
     def setUp(self):
         mock_cls = OpenStackIdentity_2_0_Connection_VOMSMockHttp
         mock_cls.type = None
-        OpenStackIdentity_2_0_Connection_VOMS.conn_classes = (mock_cls, mock_cls)
+        OpenStackIdentity_2_0_Connection_VOMS.conn_class = mock_cls
 
         self.auth_instance = OpenStackIdentity_2_0_Connection_VOMS(auth_url='http://none',
                                                                    user_id=None,
@@ -731,6 +753,29 @@ class OpenStackIdentity_3_0_MockHttp(MockHttp):
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         raise NotImplementedError()
 
+    def _v3_auth_projects(self, method, url, body, headers):
+        if method == 'GET':
+            # get user projects
+            body = json.dumps({"projects": [{"id": "project_id"}]})
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
+
+class OpenStackIdentity_3_0_federation_projects_MockHttp(OpenStackIdentity_3_0_MockHttp):
+    fixtures = ComputeFileFixtures('openstack_identity/v3')
+    json_content_headers = {'content-type': 'application/json; charset=UTF-8'}
+
+    def _v3_OS_FEDERATION_projects(self, method, url, body, headers):
+        if method == 'GET':
+            # get user projects
+            body = json.dumps({"projects": [{"id": "project_id"}]})
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
+    def _v3_auth_projects(self, method, url, body, headers):
+        return (httplib.INTERNAL_SERVER_ERROR, body, self.json_content_headers,
+                httplib.responses[httplib.INTERNAL_SERVER_ERROR])
+
 
 class OpenStackIdentity_2_0_Connection_VOMSMockHttp(MockHttp):
     fixtures = ComputeFileFixtures('openstack_identity/v2')
